`include "defines.vh"
module alu #(
        parameter DATA_WIDTH = 8
    )(
        input  wire            [7:0] opsel,
        input  wire                  enable,
        input  wire [DATA_WIDTH-1:0] rd,
        input  wire [DATA_WIDTH-1:0] rr,
        output reg  [DATA_WIDTH-1:0] out,
        input  wire            [7:0] flags_in,
        output reg             [7:0] flags_out
    );

    /* flags_out a fost transformat in reg, pentru a putea
     * fi atribuit in interiorul unui bloc always, insa va fi
     * sintetizat tot combinational (UAL-ul nici macar nu are clk
     * drept input) */
    always @* begin
        if (enable)
            case (opsel)
            `OPSEL_ADD: begin
                    $display("Executing ADD operation! Operands are: RR - %2d, RD - %2d", rr, rd);
               {flags_out[`FLAGS_C], out} = rd + rr;
                flags_out[`FLAGS_V] = (rd[7] == 1 && rr[7] == 1 && out[7] == 0) ||
                                      (rd[7] == 0 && rr[7] == 0 && out[7] == 1);
                flags_out[`FLAGS_Z] = (out == 0);
                flags_out[`FLAGS_N] = out[7];
                flags_out[`FLAGS_S] = flags_out[`FLAGS_N] ^ flags_out[`FLAGS_V];
                flags_out[`FLAGS_H] = (rd[3] == 1 && rr[3] == 1 && out[3] == 0) ||
                                      (rd[3] == 0 && rr[3] == 0 && out[3] == 1);
                flags_out[`FLAGS_T] = flags_in[`FLAGS_T];
                flags_out[`FLAGS_I] = flags_in[`FLAGS_I];
                    $display("ADD operation completed with result: %2d", out);
            end
            `OPSEL_ADC: begin
                    $display("Executing ADC operation! Operands are: RR - %2d, RD - %2d", rr, rd);
               {flags_out[`FLAGS_C], out} = rd + rr + flags_in[`FLAGS_C];
                flags_out[`FLAGS_V] = (rd[7] == 1 && rr[7] == 1 && out[7] == 0) ||
                                      (rd[7] == 0 && rr[7] == 0 && out[7] == 1);
                flags_out[`FLAGS_Z] = (out == 0);
                flags_out[`FLAGS_N] = out[7];
                flags_out[`FLAGS_S] = flags_out[`FLAGS_N] ^ flags_out[`FLAGS_V];
                flags_out[`FLAGS_H] = (rd[3] == 1 && rr[3] == 1 && out[3] == 0) ||
                                      (rd[3] == 0 && rr[3] == 0 && out[3] == 1);
                flags_out[`FLAGS_T] = flags_in[`FLAGS_T];
                flags_out[`FLAGS_I] = flags_in[`FLAGS_I];
                    $display("ADC operation completed with result: %2d", out);
            end
            `OPSEL_SUB: begin
                    $display("Executing SUB operation! Operands are: RR - %2d, RD - %2d", rr, rd);
               {flags_out[`FLAGS_C], out} = rd - rr;
                flags_out[`FLAGS_V] = (rd[7] == 1 && rr[7] == 1 && out[7] == 0) ||
                                      (rd[7] == 0 && rr[7] == 0 && out[7] == 1);
                flags_out[`FLAGS_Z] = (out == 0);
                flags_out[`FLAGS_N] = out[7];
                flags_out[`FLAGS_S] = flags_out[`FLAGS_N] ^ flags_out[`FLAGS_V];
                flags_out[`FLAGS_H] = (rd[3] == 1 && rr[3] == 1 && out[3] == 0) ||
                                      (rd[3] == 0 && rr[3] == 0 && out[3] == 1);
                flags_out[`FLAGS_T] = flags_in[`FLAGS_T];
                flags_out[`FLAGS_I] = flags_in[`FLAGS_I];
                    $display("SUB operation completed with result: %2d", out);
            end	
            `OPSEL_AND: begin
                    $display("Executing AND operation! Operands are: RR - %2d, RD - %2d", rr, rd);
                out = rd & rr;
                flags_out[`FLAGS_V] = 0;
                flags_out[`FLAGS_Z] = (out == 0);
                flags_out[`FLAGS_N] = out[7];
                flags_out[`FLAGS_S] = flags_out[`FLAGS_N] ^ flags_out[`FLAGS_V];
                flags_out[`FLAGS_H] = flags_in[`FLAGS_H];
                flags_out[`FLAGS_C] = flags_in[`FLAGS_C];
                flags_out[`FLAGS_T] = flags_in[`FLAGS_T];
                flags_out[`FLAGS_I] = flags_in[`FLAGS_I];
                    $display("AND operation completed with result: %2d", out);
            end
            `OPSEL_XOR: begin
                    $display("Executing XOR operation! Operands are: RR - %2d, RD - %2d", rr, rd);
                out = rd ^ rr;
                flags_out[`FLAGS_V] = 0;
                flags_out[`FLAGS_Z] = (out == 0);
                flags_out[`FLAGS_N] = out[7];
                flags_out[`FLAGS_S] = flags_out[`FLAGS_N] ^ flags_out[`FLAGS_V];
                flags_out[`FLAGS_H] = flags_in[`FLAGS_H];
                flags_out[`FLAGS_C] = flags_in[`FLAGS_C];
                flags_out[`FLAGS_T] = flags_in[`FLAGS_T];
                flags_out[`FLAGS_I] = flags_in[`FLAGS_I];
                    $display("XOR operation completed with result: %2d", out);
            end
            `OPSEL_OR: begin
                    $display("Executing OR operation! Operands are: RR - %2d, RD - %2d", rr, rd);
                out = rd | rr;
                flags_out[`FLAGS_V] = 0;
                flags_out[`FLAGS_Z] = (out == 0);
                flags_out[`FLAGS_N] = out[7];
                flags_out[`FLAGS_S] = flags_out[`FLAGS_N] ^ flags_out[`FLAGS_V];
                flags_out[`FLAGS_H] = flags_in[`FLAGS_H];
                flags_out[`FLAGS_C] = flags_in[`FLAGS_C];
                flags_out[`FLAGS_T] = flags_in[`FLAGS_T];
                flags_out[`FLAGS_I] = flags_in[`FLAGS_I];
                    $display("OR operation completed with result: %2d", out);
            end
            `OPSEL_NEG: begin
                    $display("Executing NEG operation! Operands are: RR - %2d, RD - %2d", rr, rd);
                out = ~rd;
                flags_out[`FLAGS_V] = 0;
                flags_out[`FLAGS_Z] = (out == 0);
                flags_out[`FLAGS_N] = out[7];
                flags_out[`FLAGS_S] = flags_out[`FLAGS_N] ^ flags_out[`FLAGS_V];
                flags_out[`FLAGS_H] = flags_in[`FLAGS_H];
                flags_out[`FLAGS_C] = flags_in[`FLAGS_C];
                flags_out[`FLAGS_T] = flags_in[`FLAGS_T];
                flags_out[`FLAGS_I] = flags_in[`FLAGS_I];
                    $display("NEG operation completed with result: %2d", out);
            end
            `OPSEL_NONE: begin
                    $display("Executing NONE operation! Operands are: RR - %2d, RD - %2d", rr, rd);
                out = rr;
                flags_out = flags_in;
                    $display("NONE operation completed with result: %2d", out);
            end
            /* Pentru a evita generarea unui latch, flags_out
             * trebuie sa ia o valoare si aici. Este de datoria noastra
             * sa nu salvam aceasta valoare in control_unit */
            default: begin
                out = {DATA_WIDTH{1'bx}};
                flags_out = flags_in;
            end
            endcase
        else /* if alu_enable == false */ begin
            out = {DATA_WIDTH{1'bx}};
            flags_out = flags_in;
        end
    end

endmodule
