/* In order to disable implicit declaration of wires */
`default_nettype none

/* SREG flags */
`define FLAGS_C             3'd0
`define FLAGS_Z             3'd1
`define FLAGS_N             3'd2
`define FLAGS_V             3'd3
`define FLAGS_S             3'd4
`define FLAGS_H             3'd5
`define FLAGS_T             3'd6
`define FLAGS_I             3'd7

/* Indirect registers */
`define XL                  5'd26
`define XH                  5'd27
`define YL                  5'd28
`define YH                  5'd29
`define ZL                  5'd30
`define ZH                  5'd31

/* IO addresses */
`define SREG                6'h3F
`define SPH                 6'h3E
`define SPL                 6'h3D
`define PUEB                6'h07
`define PORTB               6'h06
`define DDRB                6'h05
`define PINB                6'h04
`define PUEA                6'h03
`define PORTA               6'h02
`define DDRA                6'h01
`define PINA                6'h00

/* States for FSM */
`define STATE_IF            5'b00001
`define STATE_ID            5'b00010
`define STATE_EX            5'b00100
`define STATE_MEM           5'b01000
`define STATE_WB            5'b10000
`define STATE_RESET         5'b00000

/* Opcode types deduced by decoder */
`define TYPE_NOP            0
`define TYPE_UNKNOWN        1
`define TYPE_ADD            2
`define TYPE_ADC            3
`define TYPE_NEG            4
`define TYPE_AND            5
`define TYPE_OR             6
`define TYPE_EOR            7
`define TYPE_SUB            8
`define TYPE_SUBI           9
`define TYPE_LDD_Y          10
`define TYPE_LDS            11
`define TYPE_STS            12
`define TYPE_LDI            13
`define TYPE_MOV            14
`define TYPE_BRBS           15
`define TYPE_BRBC           16
`define TYPE_RJMP           17
`define TYPE_RCALL          18
`define TYPE_RET            19
`define TYPE_POP            20
`define TYPE_PUSH           21
`define TYPE_IN             22
`define TYPE_OUT            23
`define TYPE_CBI            24
`define TYPE_SBI            25

/* Opcode groups */
`define GROUP_ALU            0
`define GROUP_ALU_ONE_OP     1
`define GROUP_ALU_TWO_OP     2
`define GROUP_REGISTER       3
`define GROUP_LOAD_INDIRECT  4
`define GROUP_LOAD_DIRECT    5
`define GROUP_LOAD           6
`define GROUP_STORE_INDIRECT 7
`define GROUP_STORE_DIRECT   8
`define GROUP_STORE          9
`define GROUP_STACK          10
`define GROUP_MEMORY         11
`define GROUP_CONTROL_FLOW   12
`define GROUP_IO_READ        13
`define GROUP_IO_WRITE       14
`define GROUP_TWO_CYCLE_MEM  15
`define GROUP_ALU_AUX        16

/* Operations permitted by ALU */
`define OPSEL_NONE          0
`define OPSEL_ADD           1
`define OPSEL_ADC           2
`define OPSEL_SUB           3
`define OPSEL_AND           4
`define OPSEL_XOR           5
`define OPSEL_OR            6
`define OPSEL_NEG           7

/* Control signals */
`define CONTROL_MEM_READ        0
`define CONTROL_MEM_WRITE       1
`define CONTROL_REG_RR_READ     2
`define CONTROL_REG_RD_READ     3
`define CONTROL_REG_RD_WRITE    4
`define CONTROL_IO_READ         5
`define CONTROL_IO_WRITE        6
`define CONTROL_PREINC          7
`define CONTROL_POSTDEC         8